;;; tdbsf-db-file-mode.el --- TDBSF Database File mode

;; Copyright (c) 2001 Florent Rougon

;; Maintainer: Florent Rougon <flo@via.ecp.fr>

;; This file is part of TDBSF.

;; TDBSF is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; TDBSF is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with TDBSF; see the file COPYING. If not, write to the Free
;; Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

;;; (defvar tdbsf-narrow-to-displayed-record nil
;;;  "XXXdoesn't work yet.")

(define-derived-mode tdbsf-db-file-mode text-mode "TDBSF Database File"
  "Major mode for displaying and editing of TDBSF db files.

This mode is derived from Text mode.

It is automatically entered in the appropriate buffer when you select a
record match in the *TDBSF Results Summary* buffer.
\\{tdbsf-db-file-mode-map}"
  (mapcar 'make-local-variable
          '(tdbsf-this-file-s-records-info
            tdbsf-matches-overlays
            tdbsf-session-id)))

(defun tdbsf-create-this-file-s-records-info-and-markers
  (record-info-array buf gri)
  "XXXmarkers created; array referencing marker + GRI for each record;
session id"
  ;; GRI = Global Record Index
  ;; The value of gri on entry of this function is for the first matching
  ;; record of the current file (the one that buf is visiting).
  (with-current-buffer buf
    ;; These bindings are intended to be local to the buffer buf
    ;; (responsability of the caller)
    (setq tdbsf-session-id
          (with-current-buffer tdbsf-summary-buffer-name tdbsf-session-id)
          tdbsf-matches-overlays ()
          tdbsf-this-file-s-records-info
          (make-vector (length record-info-array) nil))
    ;; `newgri' is simply better suited than `gri' for the algorithm
    (let ((i -1)
          (newgri (1- gri))
          (loop-count (length record-info-array)))
      (while (< (setq i (1+ i)) loop-count)
        (aset tdbsf-this-file-s-records-info i
              (vector
               (set-marker (make-marker)
                           ;; We set the marker at the beginning of the "boh"
                           ;; part of the header
                           (aref (car (aref record-info-array i)) 0))
               (setq newgri (1+ newgri))
               ))
        ))))

(defun tdbsf-fontify-matches (record-info-array buf)
  (require 'tdbsf-db-file-faces)
  (let* ((and-faces (make-vector tdbsf-db-file-and-faces-count nil))
         (near-faces (make-vector tdbsf-db-file-near-faces-count nil))
         face-names-array
         face-name-index
         and-face-index
         near-face-index
         (i -1)
         overlay)
    ;; Generate face names arrays
    (while (< (setq i (1+ i)) tdbsf-db-file-and-faces-count)
      (aset and-faces i (intern (concat "tdbsf-db-file-and-face-"
                                        (number-to-string (1+ i))))))
    (setq i -1)
    (while (< (setq i (1+ i)) tdbsf-db-file-near-faces-count)
      (aset near-faces i (intern (concat "tdbsf-db-file-near-face-"
                                        (number-to-string (1+ i))))))
    (with-current-buffer buf
      ;; Generate the overlays and give them the desired properties
      (mapcar
       #'(lambda (record-info)
           ;; Reset the face numbers for each record
           (setq and-face-index -1
                 near-face-index -1)
           (mapcar
            #'(lambda (group)
                (if (string-equal "&" (car group))
                    (setq and-face-index (mod (1+ and-face-index)
                                              tdbsf-db-file-and-faces-count)
                          face-names-array and-faces
                          face-name-index and-face-index)
                  (setq near-face-index (mod (1+ near-face-index)
                                             tdbsf-db-file-near-faces-count)
                        face-names-array near-faces
                        face-name-index near-face-index))
                (mapcar
                 #'(lambda (match-span)
                     (overlay-put
                      (setq overlay
                            (make-overlay (car match-span) (cdr match-span)))
                      'face (aref face-names-array face-name-index))
                     (overlay-put overlay 'evaporate t)
                     ;; Reference each overlay in order to be able to delete
                     ;; it in case of a new search leading to matches in the
                     ;; same file. It is better than using `overlays-in'
                     ;; because if overlays are used for another goal in the
                     ;; same buffer, we'll be able to delete selectively the
                     ;; ones used to fontify matches.
                     ;; This assignment is local to `buf'.
                     (setq tdbsf-matches-overlays
                           (cons overlay tdbsf-matches-overlays))
                     )
                 (cdr group))
                )
            (cdr record-info))
           )
       record-info-array))
    ))

(defun tdbsf-bring-record-under-the-eyes (global-record-index buf same-window)
  (let* ((global-records-index-entry
          (aref (with-current-buffer tdbsf-summary-buffer-name
                  tdbsf-records-index) global-record-index))
         (window-start
         (with-current-buffer buf
           (goto-char (marker-position
                       (aref (aref tdbsf-this-file-s-records-info
                                   (aref global-records-index-entry 2)) 0)))
           (beginning-of-line)
;           (if tdbsf-narrow-to-displayed-record
;               (narrow-to-region
;                (point) (aref (car (aref global-records-index-entry 1)) 4)))
           (point))))
    (set-window-start
     (if same-window (progn (switch-to-buffer buf) (selected-window))
       (display-buffer buf))
     window-start))
  )

(defun tdbsf-previous-record-index (pos)
  "XXXReturn the global record index."
  (interactive "d")
  "Must be called in a buffer where the local variable
`tdbsf-this-file-s-records-info' has been initialized."
  (let ((i (length tdbsf-this-file-s-records-info))
        (found nil)
        tmp)
    (while (and (not found) (>= (setq i (1- i)) 0))
      (if (<
           (marker-position (aref (aref tdbsf-this-file-s-records-info i) 0))
           pos)
          (setq found t)))
    (if found
        (aref (aref tdbsf-this-file-s-records-info i) 1)
      ;; If we didn't find a previous record in this buffer and if the global
      ;; index of the first record of this buffer is 0 (i.e., it is the first
      ;; record of the first file that matched), then return nil.
      (if (zerop (setq tmp (aref (aref tdbsf-this-file-s-records-info 0) 1)))
          nil
        (1- tmp)))
    ))

(defun tdbsf-next-record-index (pos)
  "XXXReturn the global record index."
  (interactive "d")
  "Must be called in a buffer where the local variable
`tdbsf-this-file-s-records-info' has been initialized."
  (let ((i -1)
        (max-loop-count (length tdbsf-this-file-s-records-info))
        (found nil)
        (total-number-of-matching-records
         (car (with-current-buffer tdbsf-summary-buffer-name tdbsf-results)))
        tmp)
    (while (and (not found) (< (setq i (1+ i)) max-loop-count))
      (if (>
           (marker-position (aref (aref tdbsf-this-file-s-records-info i) 0))
           pos)
          (setq found t)))
    (if found
        (aref (aref tdbsf-this-file-s-records-info i) 1)
      ;; If we didn't find a next record in this buffer and if the global
      ;; index of the last record of this buffer plus one equals the total
      ;; number of records (i.e., it is the last record of the last file that
      ;; matched), then return nil.
      (if (< (setq tmp (1+ (aref (aref tdbsf-this-file-s-records-info
                                       (1- max-loop-count)) 1)))
             total-number-of-matching-records)
          tmp
        nil)
    )))

(defun tdbsf-previous-record (pos)
  (interactive "d")
  (let ((record-index (tdbsf-previous-record-index pos)))
    (if record-index
        (tdbsf-display-record record-index t)
      (message "Already on the first record")))
  )

(defun tdbsf-next-record (pos)
  (interactive "d")
  (let ((record-index (tdbsf-next-record-index pos)))
    (if record-index
        (tdbsf-display-record record-index t)
      (message "Already on the last record")))
  )

(define-key tdbsf-db-file-mode-map "\C-c\C-p" 'tdbsf-previous-record)
(define-key tdbsf-db-file-mode-map "\C-c\C-n" 'tdbsf-next-record)

(provide 'tdbsf-db-file-mode)
