;;; tdbsf-user-interface.el --- TDBSF ELisp end-user interface

;; Copyright (c) 2001 Florent Rougon

;; Maintainer: Florent Rougon <flo@via.ecp.fr>

;; This file is part of TDBSF.

;; TDBSF is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; TDBSF is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with TDBSF; see the file COPYING. If not, write to the Free
;; Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(require 'tdbsf)

(defvar tdbsf-default-pattern
  "/home/flo/src/tdbsf/db_papa/*.asc")

(defvar tdbsf-default-search-string
  "header or body"
  "XXX")

(defvar tdbsf-default-header-body-separator
  ""
  "XXX")

(defvar tdbsf-boh-pattern
  "^ *{60,} *\n"
  "A pattern identifying \"Trouve\"'s ugly beginning of header string.")

(defvar tdbsf-boh-flags
  "MULTILINE"
  "The flags going with `tdbsf-boh-pattern' to define a regular
expression for the \"re\" Python module.")

(defvar tdbsf-eoh-pattern
  "^ *{60,} *\n"
  "A pattern identifying \"Trouve\"'s ugly end of header string.")

(defvar tdbsf-eoh-flags
  "MULTILINE"
  "The flags going with `tdbsf-eoh-pattern' to define a regular
expression for the \"re\" Python module.")

(defvar tdbsf-search-string-alist
  '(("header" . "header")
    ("body" . "body")
    ("header or body" . "header_or_body")
    ("header + custom separation string + body (all concatenated)"
     . "header_plus_body"))
  "XXX")

(defvar tdbsf-db-files-mode
  "binary"
  "XXX")

(defvar tdbsf-crlf-to-lf-in-args
  t
  "XXX")

(defvar tdbsf-dont-consider-eol-as-lf
  nil
  "XXX")

(defun tdbsf (expr pattern search-string header-body-separator)
  "XXX"
  (interactive
   (let*
       (;; Pattern
        (pattern
         (let ((insert-default-directory nil))
           (expand-file-name (read-file-name "Pattern: " nil nil nil
                                             tdbsf-default-pattern))))
        ;; Search string ("header", "header_plus_body", etc.)
        (search-string
         (cdr (assoc (completing-read
                      (concat "What part of a record should \"match\": (default "
                              tdbsf-default-search-string ") ")
                      tdbsf-search-string-alist
                      nil t nil nil tdbsf-default-search-string)
                     tdbsf-search-string-alist)))
        ;; Separator string to insert in the "header_plus_body" case
        (header-body-separator
         (if (string-equal search-string "header_plus_body")
         (read-string
          (concat 
           "Separator string to insert between header and body: (default "
           tdbsf-default-header-body-separator ") ")
          nil nil tdbsf-default-header-body-separator)
         tdbsf-default-header-body-separator ; does not matter
         )))
     (list (buffer-substring-no-properties (region-beginning) (region-end))
           pattern search-string header-body-separator)))
  (call-tdbsf expr pattern tdbsf-boh-pattern tdbsf-boh-flags
              tdbsf-eoh-pattern tdbsf-eoh-flags
              search-string header-body-separator tdbsf-db-files-mode
              tdbsf-crlf-to-lf-in-args tdbsf-dont-consider-eol-as-lf))

(provide 'tdbsf-user-interface)
